import {
  Loader, Metadata, IDictionary, IEventParam, Http, I18n, Datasource, Config, DatasourceFactory,
} from '@zeedhi/core';

import {
  GridEditable, Form, Select, DialogService, Card, Button, Column, Input,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme12000GrupoPrato {
  public appController = Loader.getInstance('AppController');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  public newRow = false;
  
  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private pratoIsVisible = false;
  
  public selectedGrupo = '';
  
  public padraoGrupoPrato = [];
  
  public cdpratoComponentProps = {
    component: 'ZdSelectMultiple',
    events: {
      focus: '{{Pme12000GrupoPrato.setFilterPrato}}',
    },
    dataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    formatterDataText: [
      {
        name: 'CDPRATO',
        mask: (Config.masks.plateMask as PmeMask).pattern,
      },
      'NMPRATO',
    ],
    dataValue: 'CDPRATO',
    returnObject: false,
    dense: true,
    dataValueOut: [
      {
        column: 'CDFILAUXPRAT',
        columnOnGrid: 'CDFILAUXPRAT',
      },
    ],
    datasource: {
      uniqueKey: 'PK',
      route: '/find/pratoNivelCompleto',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    let valueCard;
    
    if (cdtipoCard.returnObject === true && cdtipoCard.value !== null) valueCard = cdtipoCard.value.CDTIPOCARD;
    else valueCard = cdtipoCard.value;
    
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: valueCard,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterAgrup() {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const cdservfili = Metadata.getInstance<Select>('SERVICO');
    const agrup = Metadata.getInstance<Select>('AGRUP');
    const agrupamento = Metadata.getInstance<GridEditable>('AGRUPAMENTO');
    
    if (cdtipoCard.value && cdservfili.value) {
      const conditions = [
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: cdtipoCard.value ? cdtipoCard.value : '',
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: cdservfili.value ? cdservfili.value : '',
        },
        {
          name: 'NRAGRUPGRPRAT',
          operator: '=',
          value: agrup.value ? agrup.value : '',
        },
      ];
      agrupamento.datasource.filter = conditions;
      agrupamento.datasource.get();
      agrupamento.editing = false;
    } else {
      agrupamento.datasource.data = [];
      agrupamento.datasource.filter = [];
    }
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const servico = Metadata.getInstance<Select>('SERVICO');
    const agrup = Metadata.getInstance<Select>('AGRUP');
    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      agrup.setValue('');
      this.closeCardPrato();
    }
  }
  
  get gridColsReturnValue(): number {
    return this.pratoIsVisible ? 6 : 12;
  }
  
  public confirmDeleteGrupo({ component }: IEventParam<Button>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiGrupo',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteGrupo() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public cancelEditingTelaGrupo({ component }: IEventParam<Button>) {
    this.gridEditableController.cancelEditing({ component });
  }
  
  public cancelEditingTelaPrato({ component }: IEventParam<Button>) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    if (grid.editing) {
      return this.gridEditableController.defaultState(grid).then((result: IDictionary) => {
        let i = 0;
        result.forEach((prat: IDictionary) => {
          grid.datasource.data[i].CDFILAUXPRAT = [prat.CDFILAUXPRAT];
          grid.datasource.data[i].CDPRATO = [prat.CDPRATO];
          grid.datasource.data[i].NMPRATO = [prat.NMPRATO];
          i += 1;
        });
      });
    }
    return false;
  }
  
  public deleteGrupo() {
    const gridAgrup = Metadata.getInstance<GridEditable>('AGRUPAMENTO');
    const checkedRows = gridAgrup.selectedRows;
    Http.post('/excluiGrupo', { requestType: 'Row', checkedRows }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        gridAgrup.datasource.get();
        gridAgrup.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
        DialogService.hide();
      } else {
        const mensagem = response.data.dataset.mensagem[0];
        if (mensagem.indexOf('ERROR_TEM_PRATO') === 0) {
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_TEM_PRATO'), 'error');
          DialogService.hide();
        } else if (mensagem.indexOf('ERROR_TEM_ITEM_FREQUENCIA') === 0) {
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_TEM_ITEM_FREQUENCIA'), 'error');
          DialogService.hide();
        } else if (mensagem.indexOf('ERROR_TEM_ITEM_RESTRICAO') === 0) {
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_TEM_ITEM_RESTRICAO'), 'error');
          DialogService.hide();
        } else {
          this.utilities.showAlert('alertError', `pme::${mensagem}`, 'error');
          DialogService.hide();
        }
      }
    }).catch((error) => {
      gridAgrup.selectedRows = [];
      this.utilities.showAlert('alertError', error, 'error');
      DialogService.hide();
    }).finally(() => {
      this.closeCardPrato();
    });
  }
  
  public save({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formAgrup');
    const grid: GridEditable = Metadata.getInstance('AGRUPAMENTO');
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const cdservfili = Metadata.getInstance<Select>('SERVICO');
    const editedRows = grid.getEditedRows(true);
    
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.CDTIPOCARD = cdtipoCard.value;
        row.CDSERVFILI = cdservfili.value;
        // row.NRGRUPOPRATOPAE = nrgrupo.value ? nrgrupo.value : row.NRGRUPOPRATOPAE;
        // row.NMGRUPOPRATOPAE = row.CDGRUPOPRATOPAE !== undefined ? row.CDGRUPOPRATOPAE : row.NMGRUPOPRATOPAE;
      });
      
      if (grid) grid.datasource.loading = true;
      Http.post('/salvaGrupo', { editedRows }).then((response: IDictionary) => {
        if (response.data.length === 0) {
          grid.datasource.get();
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          this.gridEditableController.cancelEditing({ component });
        } else {
          const mensagem = response.data.dataset.mensagem[0];
          let param = '';
          if (mensagem.indexOf('ERROR_GRUPO_CADASTRADO') === 0) {
            param = mensagem.split(':');
            const err = 'pme::ERROR_GRUPO_CADASTRADO';
            this.utilities.showAlert('alertError', I18n.translate(err, { PARAM1: param[1] }), 'error');
          } else {
            this.utilities.showAlert('alertError', mensagem, 'error');
          }
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      }).finally(() => {
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  public closeCardPrato() {
    const card = Metadata.getInstance<Card>('pratoCard');
    const gridPrato: GridEditable = Metadata.getInstance('PRATO');
    this.gridEditableController.cancelEditing({ component: gridPrato });
    
    card.isVisible = false;
    this.pratoIsVisible = false;
  }
  
  private cardPratoVisible() {
    const card = Metadata.getInstance<Card>('pratoCard');
    try {
      if (!this.gridEditableController.newRow) {
        card.isVisible = true;
        this.pratoIsVisible = true;
      } else {
        const btn = Metadata.getInstance<Button>('excluiPrato');
        this.cancelEditingTelaPrato({ component: btn }).then(() => {
          card.isVisible = true;
          this.pratoIsVisible = true;
        });
      }
    } catch (Error) {
      card.isVisible = false;
      this.pratoIsVisible = false;
    }
  }
  
  public async carregaPrato({ component }: IEventParam<Datasource>) {
    const grupoSelecionado = !!component.currentRow.NRGRUPOPRATOPAE;
    const card = Metadata.getInstance<Card>('pratoCard');

    if (grupoSelecionado) {
      const prato: GridEditable = Metadata.getInstance('PRATO');
      prato.selectedRows = [];
      // prato.columns[0].loading = true;
      this.selectedGrupo = `${component.currentRow.CDGRUPOPRATOPAE} - ${component.currentRow.NMGRUPOPRATOPAE}`;
    
      const conditions = [
        {
          name: 'NRGRUPOPRATOPAE',
          operator: '=',
          value: component.currentRow.NRGRUPOPRATOPAE,
        },
      ];
      prato.datasource.filter = conditions;
      prato.datasource.loading = true;
      const cloneConfig = prato.datasource.clone();
      const cloneDs = DatasourceFactory.factory(cloneConfig);
      await cloneDs.get().then(() => {
        for (let i = 0; i < cloneDs.data.length; i += 1) {
          cloneDs.data[i].CDFILAUXPRAT = [cloneDs.data[i].CDFILAUXPRAT];
          cloneDs.data[i].CDPRATO = [cloneDs.data[i].CDPRATO];
          cloneDs.data[i].NMPRATO = [cloneDs.data[i].NMPRATO];
        }
        prato.datasource.updateData(cloneDs.data);
        prato.datasource = cloneDs;
        this.cardPratoVisible();
      });
    } else {
      this.selectedGrupo = `${component.currentRow.CDGRUPOPRATOPAE} - ${component.currentRow.NMGRUPOPRATOPAE}`;
      card.isVisible = false;
      this.pratoIsVisible = false;
    }
  }
  
  public salvaPrato({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formPrato');
    const grid: GridEditable = Metadata.getInstance('PRATO');
    const editedRows = grid.getEditedRows();
    const grupo: GridEditable = Metadata.getInstance('AGRUPAMENTO');
    const currentRow = grupo.datasource.currentRow;
    const cdservfili = Metadata.getInstance<Select>('SERVICO');

    if (!currentRow.CDGRUPOPRATOPAE) this.utilities.showAlert('alertError', 'pme::grupoNaoSelecionado', 'error');
    else if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.NRGRUPOPRATOPAE = grupo.datasource.currentRow.NRGRUPOPRATOPAE;
        row.CDSERVFILI = cdservfili.value;
      });
      
      if (grid) grid.datasource.loading = true;
      Http.post('/salvaPrato', { editedRows }).then((response: IDictionary) => {
        let param;
        if (response.data.length === 0) {
          // grid.datasource.get();
          // this.reloadPrato('');
          
          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'success');
          // this.gridEditableController.cancelEditing({ component });
          // this.cancelEditingTelaPrato({ component });
        } else if (response.data.dataset.mensagem[0].indexOf('ERROR_PRAT_CAD') === 0) {
          param = response.data.dataset.mensagem[0].split(':');
          this.utilities.showAlert('alertError', I18n.translate('pme::ERROR_PRAT_CAD', { PARAM1: param[1] }), 'error');
          // this.gridEditableController.cancelEditing({ component });
        }
      }).catch((error) => {
        this.utilities.showAlert('alertError', error, 'error');
      }).finally(() => {
        // this.reloadPrato('');
        this.cancelEditingTelaPrato({ component });
        if (grid) grid.datasource.loading = false;
      });
    }
  }
  
  public returnDadosPadraoGrupo() {
    const component: GridEditable = Metadata.getInstance('AGRUPAMENTO');
    Http.get('/find/padraoGrupoPrato').then((response) => {
      const result = response.data.data;
      this.padraoGrupoPrato = result;
      const nmGrupoPratoPae = this.utilities.returnColumn(component, 'GRUPOPRATOPAE') as Column;
      const nrAgrupPrat = this.utilities.returnColumn(component, 'NRAGRUPGRPRAT') as Column;
      const nmGrupo = this.utilities.returnColumn(component, 'NMGRUPOPRATOPAE') as Column;
      const nrgrupoprato = this.utilities.returnColumn(component, 'CDGRUPOPRATOPAE') as Column;
      if (result.length > 0) {
        nmGrupoPratoPae.isVisible = true;
        nrAgrupPrat.isVisible = true;
        nmGrupo.isVisible = false;
        nrgrupoprato.isVisible = false;
      } else {
        nmGrupoPratoPae.isVisible = false;
        nrAgrupPrat.isVisible = true;
        nmGrupo.isVisible = true;
        nrgrupoprato.isVisible = true;
      }
    });
  }
  
  public addRowGrupo({ component }: IEventParam<GridEditable>) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const agrup = Metadata.getInstance<Select>('AGRUP');
    
    if (agrup.value) {
      const item: IDictionary = {};
      item.NRAGRUPGRPRAT = agrup.value;
      item.__is_new = true;
      this.gridEditableController.newRow = true;
      grid.addNewRow(item, 'start');
      if (!this.gridEditableController.isInlineEditing) this.gridEditableController.onInlineEdit({ component: grid });
    } else this.gridEditableController.addRow({ component: grid });
    
    this.returnDadosPadraoGrupo();
  }
  
  public setFilterPrato({ component }: IEventParam<Select>) {
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    component.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: nivel,
      },
    ];
    component.datasource.get();
  }
  
  public async reloadPrato(search: String) {
    const prato: GridEditable = Metadata.getInstance('PRATO');
    const grupo: GridEditable = Metadata.getInstance('AGRUPAMENTO');
    
    const conditions = [
      {
        name: 'NRGRUPOPRATOPAE',
        operator: '=',
        value: grupo.datasource.currentRow.NRGRUPOPRATOPAE,
      },
    ];
    prato.datasource.filter = conditions;
    if (typeof search === 'string') prato.datasource.search = search;
    else prato.datasource.search = '';
    prato.datasource.loading = true;
    const cloneConfig = prato.datasource.clone();
    const cloneDs = DatasourceFactory.factory(cloneConfig);
    await cloneDs.get().then(() => {
      for (let i = 0; i < cloneDs.data.length; i += 1) {
        cloneDs.data[i].CDFILAUXPRAT = [cloneDs.data[i].CDFILAUXPRAT];
        cloneDs.data[i].CDPRATO = [cloneDs.data[i].CDPRATO];
        cloneDs.data[i].NMPRATO = [cloneDs.data[i].NMPRATO];
      }
      prato.datasource.updateData(cloneDs.data);
      prato.datasource = cloneDs;
    });
  }
  
  public async deletePrato({ component }: IDictionary) {
    const grid: GridEditable = Metadata.getInstance('PRATO');
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      await DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'pme::sim',
            events: {
              click: () => {
                const buttonDel: GridEditable = Metadata.getInstance('yesDelete');
                this.utilities.activeLoadingButton(buttonDel);
                this.crudController.deleteRows(checkedRows, 'pratosGrupoPrato').then(() => {
                  // grid.cancelEditedRows();
                  // grid.datasource.get();
                  this.reloadPrato('');
                  // grid.selectedRows = [];
                  this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
                }).catch((error: string) => {
                  this.utilities.showAlert('alert', `pme::${error}`, 'error');
                }).finally(() => {
                  this.utilities.disableLoadingButton(buttonDel);
                  DialogService.hide();
                });
              },
            },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public searchPrato({ component }: IEventParam<Input>) {
    if (typeof component.value === 'string') this.reloadPrato(component.value);
    else this.reloadPrato('');
  }
  
  public editGroupName({ row }: IDictionary) {
    if (row.__is_new) return true;
    if (this.padraoGrupoPrato.length > 0) return false;
    return true;
  }
}
